#!/usr/bin/env python

import os, sys, optparse, tarfile, platform

script_dir = os.path.dirname(os.path.abspath(sys.argv[0]))
tmp_path = os.path.join(script_dir, "tmp")
usage = "%s [options] DEB" % os.path.basename(sys.argv[0])
p = optparse.OptionParser(
	description='Takes a debian package and makes info files based on it',
	prog='deb-process',
	usage=usage + "\nwhere DEB is the package to process if --all/-a is not enabled,\nor the directory whose packages to process if it is enabled.")
p.add_option("--all", "-a", default="store_false", action="store_true", help="process all debs in the current working directory.  If this is not enabled, you must specify a DEB (.deb file) to process.  If it is, DEB can be a directory where the debs are.")
p.add_option("--output", "-o", default=os.path.join(script_dir, "files"), help="directory to save the resulting files to to.  Defaults to files/ in the script's directory.")
p.add_option("--template-path", "-t", default=os.path.join(script_dir, "templates"), help="directory where your templates are located.  Defaults to templates/ in the script's directory.")
options, args = p.parse_args()

def readfile(thefile, lines=False):
	fileobj = open(thefile)
	if lines == False:
		spam = fileobj.read()
	else:
		spam = fileobj.readlines()
	fileobj.close()
	return spam

if os.path.exists(options.output) != True:
	os.mkdir(options.output)
if os.path.exists(tmp_path) != True:
	os.mkdir(tmp_path)

if len(args) != 1 and options.all != True:
	print "error: no DEB given and --all/-a is not enabled"
	p.print_help()
	sys.exit(2)

if options.all == True and len(args) == 1:
	deb_dir = args[0]
elif options.all == True and len(args) < 1:
	deb_dir = os.getcwd()

debs = []
templates = []
if options.all == True:
	dirlist = os.listdir(deb_dir)
	for i in dirlist:
		use = os.path.abspath(os.path.join(deb_dir, i))
		if use.endswith(".deb"):
			debs.append(use)
	debs.sort()
else:
	debs.append(args[0])

dirlist2 = os.listdir(options.template_path)
for i in dirlist2:
	use = os.path.join(options.template_path, i)
	if os.path.isdir(use) != True:
		templates.append(use)

for i in debs:
	fields = {}
	templatetags = {}
	basename = os.path.basename(i)
	cwd = os.getcwd()
	os.chdir(tmp_path)
	if platform.system() == "Windows":
		extract_control_tar = "%s x %s control.tar.gz" % (os.path.join(script_dir, "ar.exe"), i)
	else:
		extract_control_tar = "ar x %s control.tar.gz" % i
	os.system(extract_control_tar)
	os.chdir(cwd)
	tar = tarfile.open(os.path.join(tmp_path, "control.tar.gz"))
	if "control" in tar.getnames():
		tar.extract("control", tmp_path)
	else:
		tar.extract("./control", tmp_path)
	tar.close()
	controllines = readfile(os.path.join(tmp_path, "control"), lines=True)
	os.remove(os.path.join(tmp_path, "control.tar.gz"))
	os.remove(os.path.join(tmp_path, "control"))
	last = ""
	for c in controllines:
		if c[0] != " ":
			d = c.split(": ", 1)
			fields[d[0]] = d[1].strip() + "\n"
			last = d[0]
		else:
			fields[last] = fields[last] + c.strip() + " "
	for i in fields:
		fields[i] = fields[i].strip()
		if i == "Author":
			templatetags["author name"] = fields[i].rsplit(" <", 1)[0]
			templatetags["author email"] = fields[i].rsplit(" <", 1)[1].strip(">").strip()
		if i == "Description":
			if "\n" in fields[i]:
				templatetags["description short"] = fields[i].split("\n", 1)[0]
				templatetags["description long"] = fields[i].split("\n", 1)[1]
			else:
				templatetags["description short"] = fields[i]
				templatetags["description long"] = ""
		if i == "Version":
			if "-" in fields[i]:
				templatetags["friendly_version"] = fields[i].rsplit("-", 1)[0]
		templatetags["field %s" % i] = fields[i]
	templatetags["debname"] = basename.rsplit(".deb", 1)[0]
		
	for k in templates:
		string = readfile(k)
		outname = os.path.basename(k)
		for j in templatetags:
			string = string.replace("{%s}" % j, templatetags[j])
			outname = outname.replace("(%s)" % j, templatetags[j])
		string = string.replace("{{}", "{")
		outname = outname.replace("(()", "(")
		outpath = os.path.join(options.output, outname)
		fileobj = open(outpath, "w")
		fileobj.write(string)
		fileobj.close()
